import unittest
from v8unpack import helper
import os
import time
from multiprocessing import Pool, Manager, Lock


def test_write(params):
    data, path, file_name = params
    _path = os.path.join(path, file_name)
    os.makedirs(path, exist_ok=True)
    with open(_path, 'w', encoding='utf-8') as file:
        time.sleep(1)
        print(data)
        file.write(data)
    return data


class TestLockFiles(unittest.TestCase):
    def test_multi_write(self):
        pool = helper.get_pool(processes=3)
        tasks = []
        path = os.path.join(os.path.dirname(__file__), 'tmp')
        for i in range(30):
            tasks.append([f'test{i}', path, 'test.txt'])

        with pool:
            res = pool.map(test_write, tasks, chunksize=10)
        pass


from .Form803 import Form803


class TaskForm803(Form803):
    @classmethod
    def get_decode_obj_header(cls, header):
        return header[0]

    def encode_header(self):
        return [[
            "1",
            self.encode_header_title(),
            "0"
        ]]



from .Form803 import Form803


class ReportForm803(Form803):
    @classmethod
    def get_decode_obj_header(cls, header):
        return header[0][1][1]

    # @classmethod
    # def get_decode_header(cls, header_data):
    #     return header_data[0][1][1][1][1]

    def encode_header(self):
        return [[
            "1",
            [
                "1",
                [
                    "0",
                    self.encode_header_title(),
                ],
                "0"
            ]]]



import os
from base64 import b64decode, b64encode

from .. import helper
from ..MetaDataObject.core.Simple import Simple
from ..ext_exception import ExtException


class CommonPicture(Simple):
    def __init__(self):
        super(Simple, self).__init__()
        self.data = None
        self.raw_data = None

    def decode_object(self, src_dir, file_name, dest_dir, dest_path, version, header_data):
        try:
            super().decode_object(src_dir, file_name, dest_dir, dest_path, version, header_data)
            try:
                self.header['info'] = helper.json_read(src_dir, f'{self.header["uuid"]}.0.json')
            except FileNotFoundError:
                return
            if self.header['info'][0][2] and self.header['info'][0][2][0] and self.header['info'][0][2][0][0]:
                bin_data = self._extract_b64_data(self.header['info'][0][2][0])

                extension = helper.get_extension_from_comment(self.header['comment'])
                _dest_dir = os.path.join(dest_dir, dest_path)
                if dest_dir:
                    helper.bin_write(bin_data, _dest_dir, f'{self.header["name"]}.{extension}')
        except Exception as err:
            raise ExtException(parent=err)

    def encode_object(self, src_dir, file_name, dest_dir, version):
        super(CommonPicture, self).encode_object(src_dir, file_name, dest_dir, version)

        extension = helper.get_extension_from_comment(self.header['comment'])
        try:
            bin_data = helper.bin_read(src_dir, f'{self.header["name"]}.{extension}')
            self.header['info'][0][2][0][0] += b64encode(bin_data).decode(encoding='utf-8')
            helper.json_write(self.header['info'], dest_dir, f'{self.header["uuid"]}.0.json')
        except FileNotFoundError:
            pass



import os
import sys
import unittest

sys.path.append("../../src/")
from v8unpack.MetaDataObject.CommonForm import CommonForm
from v8unpack.MetaDataObject.versions.Template803 import Template803 as Meta

class TestFileOrganizerCE(unittest.TestCase):
    def setUp(self) -> None:
        self.current_dir = os.path.dirname(__file__)
        self.data_dir = os.path.join(self.current_dir, 'data')
        self.temp_dir = os.path.join(self.data_dir, 'temp')

    def test_near(self):
        form = Meta()
        form.decode(self.data_dir, "cb225ac1-0acf-4807-aae2-0e135699ea05", self.temp_dir, "cb225ac1-0acf-4807-aae2-0e135699ea05", 803)



import os
import sys
import unittest

sys.path.append("../../src/")
from v8unpack import helper
from v8unpack.MetaDataObject.versions.Form803 import Form803
from v8unpack.unittest_helper import compare_file, NotEqualLine


class TestFileOrganizerCE(unittest.TestCase):
    def setUp(self) -> None:
        self.current_dir = os.path.dirname(__file__)
        self.data_dir = os.path.join(self.current_dir, 'data/form_elem_803')
        self.temp_dir = os.path.join(self.data_dir, 'temp')
        self.temp_dir2 = os.path.join(self.data_dir, 'temp2')

    def test_decode_all_form_elem(self):
        test_files = os.listdir(self.data_dir)
        result = ''
        for file_name in test_files:
            if file_name == 'temp':
                continue
            uuid, x, y = file_name.split('.')
            result += self.decode_form_elem(uuid)
        self.assertEqual('', result)

    def test_decode_form_elem(self):
        uuid = '29ecd407-e446-407e-8321-6de183302fd6'
        result = self.decode_form_elem(uuid)
        self.assertEqual('', result)

    def decode_form_elem(self, uuid):
        file_name = f'{uuid}.0.json'
        helper.clear_dir(self.temp_dir)
        form803 = Form803()
        form803.new_dest_dir = self.temp_dir
        form803.form = [helper.json_read(self.data_dir, file_name)]
        form803.decode_includes(None, self.temp_dir, '', None)

        form803.write_decode_object(self.temp_dir, '', uuid, 803)
        form803.encode_includes(self.temp_dir, uuid, self.temp_dir, 803)
        helper.json_write(form803.form[0], self.temp_dir, f'{uuid}.0.json')
        problems = ''
        try:
            result = compare_file(
                os.path.join(self.data_dir, file_name),
                os.path.join(self.temp_dir, file_name),
                problems
            )
        except NotEqualLine as err:
            result = err
        if result:
            print(result)
        return result



from ..MetaDataObject.core.Simple import SimpleNameFolder
from ..MetaDataObject.versions.TaskForm803 import TaskForm803


class ChartOfCharacteristicTypeForm(SimpleNameFolder):
    versions = {
        '803': TaskForm803
    }



import os
import sys
import unittest

sys.path.append("../../src/")
from v8unpack import helper
from v8unpack.json_container_decoder import JsonContainerDecoder
from v8unpack.unittest_helper import compare_file, NotEqualLine


class TestFileOrganizerCE(unittest.TestCase):
    def setUp(self) -> None:
        self.current_dir = os.path.dirname(__file__)
        self.data_dir = os.path.join(self.current_dir, 'data/json_decode_src')
        self.temp_dir = os.path.join(self.data_dir, 'temp')
        self.temp_dir2 = os.path.join(self.data_dir, 'temp2')

    def test_decode_raw_text_json_object(self):
        helper.clear_dir(self.temp_dir)
        res = JsonContainerDecoder.decode([self.data_dir, 'raw_text_json_object', self.temp_dir])
        pass

    def test_decode_raw_text_json_object_no_format(self):
        helper.clear_dir(self.temp_dir)
        res = JsonContainerDecoder.decode([self.data_dir, 'raw_text_json_object_no_format', self.temp_dir])
        pass

    def test_decode_raw_text_json_array(self):
        helper.clear_dir(self.temp_dir)
        res = JsonContainerDecoder.decode([self.data_dir, 'raw_text_json_array', self.temp_dir])
        pass

    def test_decode_raw_text_bin(self):
        helper.clear_dir(self.temp_dir)
        res = JsonContainerDecoder.decode([self.data_dir, 'raw_text_bin', self.temp_dir])
        pass

    def test_decode_raw_b64_and_string(self):
        file_name = '5f2980ca-3f1c-4680-a6e9-477825442782.0'
        helper.clear_dir(self.temp_dir)
        res1 = JsonContainerDecoder.decode([self.data_dir, file_name, self.temp_dir])
        JsonContainerDecoder.encode([self.temp_dir, f'{file_name}.json', self.temp_dir])
        problems = ''
        try:
            result = compare_file(
                os.path.join(self.data_dir, file_name),
                os.path.join(self.temp_dir, file_name),
                problems
            )
        except NotEqualLine as err:
            result = err
        print(result)



import os
import shutil
from base64 import b64encode

from ..version import __version__
from .. import helper
from ..MetaObject import MetaObject


class Configuration803(MetaObject):
    ext_code = {
        'seance': 7,
        '803': 6,
        '802': 0,
        'con': 5
    }
    help_file_number = 3

    _images = {
        'Заставка': 2
    }
    _obj_info = {
        '4': '4',
        '9': '9',
        '8': '8',
        'a': 'a',
        'b': 'b',
        'c': 'c',
        'd': 'd',
    }

    def __init__(self):
        super(Configuration803, self).__init__()
        self.counter = {}

    @classmethod
    def decode(cls, src_dir, dest_dir, *, version=None):
        self = cls()
        self.header = {}
        root = helper.json_read(src_dir, 'root.json')
        self.header['v8unpack'] = __version__
        self.header["file_uuid"] = root[0][1]
        self.header["root_data"] = root

        _header_data = helper.json_read(src_dir, f'{self.header["file_uuid"]}.json')
        self.set_header_data(_header_data)

        file_name = cls.get_class_name_without_version()

        self.header['version'] = helper.json_read(src_dir, 'version.json')
        # self.header['versions'] = helper.json_read(src_dir, 'versions.json')
        shutil.copy2(os.path.join(src_dir, 'versions.json'), os.path.join(dest_dir, 'versions.json'))

        self.decode_code(src_dir)
        self._decode_html_data(src_dir, dest_dir, 'help', header_field='help', file_number=self.help_file_number)
        self._decode_images(src_dir, dest_dir)
        self._decode_info(src_dir, dest_dir, file_name)

        helper.json_write(self.header, dest_dir, f'{file_name}.json')

        tasks = self.decode_includes(src_dir, dest_dir, '', self.header['data'])
        self.write_decode_code(dest_dir, file_name)
        return tasks
        pass

    @classmethod
    def get_decode_header(cls, header):
        return header[0][3][1][1][1][1]

    @classmethod
    def get_decode_includes(cls, header_data):
        return [
            header_data[0][3][1],
            header_data[0][4][1][1],
            header_data[0][5][1],
            header_data[0][6][1],
            header_data[0][7][1],
            header_data[0][8][1],
        ]

    @classmethod
    def encode(cls, src_dir, dest_dir, *, version=None, file_name=None, **kwargs):
        self = cls()
        helper.clear_dir(dest_dir)
        file_name = cls.get_class_name_without_version()
        self.header = helper.json_read(src_dir, f'{file_name}.json')
        helper.check_version(__version__, self.header.get('v8unpack', ''))

        helper.json_write(self.header["root_data"], dest_dir, 'root.json')
        helper.json_write(self.encode_version(), dest_dir, 'version.json')
        # helper.json_write(self.header["versions"], dest_dir, 'versions.json')
        shutil.copy2(os.path.join(src_dir, 'versions.json'), os.path.join(dest_dir, 'versions.json'))

        self._encode_html_data(src_dir, 'help', dest_dir, header_field='help', file_number=self.help_file_number)
        self._encode_images(src_dir, dest_dir)
        self.encode_code(src_dir, file_name)
        self._encode_info(src_dir, file_name, dest_dir)
        self.write_encode_code(dest_dir)
        helper.json_write(self.header['data'], dest_dir, f'{self.header["file_uuid"]}.json')
        tasks = self.encode_includes(src_dir, file_name, dest_dir, version)
        return tasks

    def encode_version(self):
        return self.header['version']

    def _decode_images(self, src_dir, dest_dir):
        if self._images:
            for elem in self._images:
                try:
                    data = helper.json_read(src_dir, f'{self.header["uuid"]}.{self._images[elem]}.json')
                except FileNotFoundError:
                    return
                try:
                    if data[0][2] and data[0][2][0] and data[0][2][0][0]:
                        bin_data = self._extract_b64_data(data[0][2][0])
                        helper.bin_write(bin_data, dest_dir, f'{elem}.bin')
                except IndexError:
                    pass
                self.header[f'image_{elem}'] = data

    def _encode_images(self, src_dir, dest_dir):
        if self._images:
            for elem in self._images:
                try:
                    bin_data = helper.bin_read(src_dir, f'{elem}.bin')
                except FileNotFoundError:
                    bin_data = None
                header = self.header.get(f'image_{elem}')
                if header and len(header[0]) > 1 and bin_data:
                    header[0][2][0][0] += b64encode(bin_data).decode(encoding='utf-8')
                if header:
                    helper.json_write(header, dest_dir, f'{self.header["uuid"]}.{self._images[elem]}.json')



from .FormElement import FormElement, calc_offset
from ....helper import FuckingBrackets


class Decoration(FormElement):
    @classmethod
    def get_name_node_offset(cls, raw_data):
        return calc_offset([(3, 1), (3, 0)], raw_data)

    @classmethod
    def decode(cls, form, raw_data):
        if raw_data[0] in ['11', '12'] and len(raw_data) != 36:
            raise FuckingBrackets()
        result = super().decode(form, raw_data)
        return result
